package Anarres::Mud::Driver::Program::Method;

use strict;
use vars qw(@ISA @EXPORT_OK);
use Data::Dumper;
use Carp qw(cluck);
use Anarres::Mud::Driver::Compiler::Lex qw(%TYPETAB);
use Anarres::Mud::Driver::Program::Variable;

@ISA = qw(Anarres::Mud::Driver::Program::Variable);

#sub new {
#	my $class = shift;
#	my $self = $class->SUPER::new(@_);
#	return $self;
#}

sub args { return $_[0]->{Args}; }

# Code is added later in the parser
sub code {
	my ($self, $code) = @_;
	# cluck "Add code $code to method $self->{Name}\n" if $code;
	$self->{Code} = $code if defined $code;
	return $self->{Code};
}

sub dump {
	my $self = shift;
	my $indent = shift;
	$indent++;

	my $sep = "\n" . ("\t" x $indent);

	my $out = "([" . $self->type->dump(@_) . "] method " . $self->name;
	my $args = join(" ", map { $_->dump($indent, @_) } @{$self->args});
	my $code = $self->code->dump($indent, @_);

	$out  = $out .
		$sep . "(args " . $args . ")" .
		$sep . $code . ")";

	return $out;
}

sub gencall {
	my ($self, @args) = @_;
	return '$self->' . $self->name . "(" . join(", ", @args) . ")";
}

	# This should generate Perl code for the method
sub generate {
	my $self = shift;
	my $indent = shift;
	$indent++;

	my $proto = '$' x @{$self->args};
	my $head =
		"\n# method " . $self->name . "\n" .
		"sub " . $self->name . " ($proto) {\n";
	my $tail = "\n}\n";

	my @args = map { $_->name } @{ $self->args };
	my $args = "\t" . 'my ($self, $' . join(', $', @args) . ') = @_;';

	return $head . $args . $self->code->generate($indent, @_) . $tail;
}

1;
